module Mailtime
  # moving the functionality to this module, you can include
  # this concern instead of sub-classing the `MailTemplate` model.
  module MailTemplateConcern

    extend ::ActiveSupport::Concern

    included do
      self.table_name = 'mailtime_mail_templates'

      belongs_to :mail_layout, :class_name => 'Mailtime::MailLayout', :foreign_key => :mailtime_mail_layout_id
      validates_presence_of :klass, :in => -> { mailers }
      validates_presence_of :format, :in => -> { Mailtime.configuration.valid_formats }
      validates_uniqueness_of :action, :scope => [:klass, :format]

      def self.mailers
        Mailtime.configuration.mailers
      end

      def self.mail_actions
        mailers.collect { |mailer| [mailer, mailer.constantize.public_instance_methods(false)] }
      end
    end

    def mailtime_mail_layout
      self.mail_layout.present? ? self.mail_layout : NullLayout.new
    end

    def additional_headers
      headers = {}
      headers[:subject] = self.subject if self.subject.present?
      headers
    end

  end
end