module Mailtime

  @@loggable_lookups = []

  def self.loggable_lookups
    @@loggable_lookups
  end

  class Configuration
    attr_accessor :mailers,
                  :render,
                  :log,
                  :log_fields,
                  :fallback,
                  :reject_scope_keys,
                  :loggable_lookups,
                  :yield_keyword,
                  :valid_formats

    def initialize
      @mailers = load_mailers
      @render = true
      @log = true
      @log_fields = [:to]
      @fallback = true
      @reject_scope_keys = ['devise_mapping', 'scope_name']
      @loggable_lookups = Mailtime.loggable_lookups
      @yield_keyword = '{{ mailtime_yield }}'
      @valid_formats = %w{html text}
    end

    def mailer(klass)
      @mailers << klass
    end

    def loggables
      @loggable_lookups.each_with_object({}) do |k,h|
        key = k.keys.first
        h[key] = k[key]
      end
    end

    def self.loggable_lookups
      @loggable_lookups ||= []
    end

    # does nothing on purpose.
    def self.loggable_lookups=(val)
      @loggable_lookups
    end

    def valid_extensions
      Mailtime::Renderers::BaseRenderer.descendants.map(&:renders).flatten
    end

    def preload_mailers!
      preload_mailers
    end

    def preload_mailers
      Dir.glob(::Rails.root.join("app/mailers/**/*_mailer.rb"))
          .each { |klass| require klass }
    end

    def load_mailers
      ObjectSpace.each_object(Class).select { |klass| klass < ::ActionMailer::Base }
    end

    protected

  end
end