module Mailtime
  class MailLogService

    attr_accessor :field, :loggable
    def initialize(mail, loggable_class, options)
      @mail = mail
      @loggable_class = loggable_class
      @loggable = nil
      @options = options
      @fields = []
    end

    def execute
      log
    end

    def log
      return unless loggable?
      create_mail_log
    end

    def loggable?
      @mail.action_variables.detect { |_, var| class_is_loggable?(var) && attribute_is_loggable?(var) } && @fields.any?
    end

    def class_is_loggable?(var)
      if var.class.to_s == @loggable_class
        @loggable = var
      end
      return false
    end

    def attribute_is_loggable?(var)
      fields = @options[:fields] || Mailtime.configuration.log_fields || [:to, :cc, :bcc]
      fields.each do |field|
        if @mail.send(field).include?(var.send(@options[:attribute]))
          @fields << field
        end
      end
    end

    def create_mail_log
      @fields.each do |field|
        ::Mailtime::MailLog.create(
            :loggable => @loggable,
            :mailtime_mail_template_id => @mail.mailtime_template.try(:id),
            :action => @mail.mailer_action,
            :klass => @mail.mailer_klass,
            :headers => extract_headers,
            :scope => extract_scope,
            :field => field
        )
      end
    end

    def extract_headers
      headers = {}
      @mail.header_fields.map { |k| headers[k.name] = k.value }
      headers
    end

    def extract_scope
      @mail.action_variables.as_json.except(*Mailtime.configuration.reject_scope_keys)
    end

  end
end