require 'mailtime/models'
require 'mailtime/renderers/mail_renderer'
require 'mailtime/renderers/base_renderer'
require 'mailtime/renderers/erb_renderer'
require 'mailtime/processor/mail_log_service'

module Mailtime
  class Processor

    CONTENT_TYPES = {
        :html => 'text/html',
        :text => 'text/plain'
    }

    def initialize(mail)
      @mail = mail
    end

    def execute
      process_mail
    end

    protected

    def process_mail
      remove_parts
      render_mail
      log_mail
      @mail
    end

    private

    def remove_part(format)
      index = @mail.parts.index { |mail_part| mail_part.content_type.index(Mailtime::Processor::CONTENT_TYPES[format])}
      @mail.parts.delete_at(index)
    end

    def remove_parts

    end

    def render_mail
      return @mail unless Mailtime.configuration.render
      renderer = Mailtime::Renderers::MailRenderer.new(@mail)
      apply_parts(renderer)
      @mail
    end

    def log_mail
      return unless Mailtime.configuration.log
      log_for_recipients
    end

    def log_for_recipients
      Mailtime.configuration.loggables.each do |loggable, options|
        svc = MailLogService.new(@mail, loggable, options)
        svc.log
      end
    end

    def apply_parts(renderer)
      [:text, :html].each do |format|
        apply_part(renderer, format)
      end
    end

    def apply_part(renderer, format)
      remove_part(format) if mail_has_template?(format)
      part = Mail::Part.new do
        content_type "#{CONTENT_TYPES[format]}; charset=UTF-8"
        body renderer.execute(format)
      end
      mthd = "#{format}_part=".to_sym
      @mail.send(mthd, part)
    end

    def mail_has_template?(format)
      !!!@mail.mailtime_templates.send(format).nil?
    end

  end
end